/**
 * Copyright (C) 2001-2005 France Telecom R&D
 */
package org.objectweb.perseus.cache;

import org.objectweb.perseus.cache.api.CacheEntryFactory;
import org.objectweb.perseus.cache.lib.BasicCacheEntryFactory;
import org.objectweb.perseus.cache.lib.BasicCacheManager;
import org.objectweb.perseus.cache.replacement.lib.AbstractReplacementManager;
import org.objectweb.perseus.cache.replacement.lib.FIFOReplacementManager;
import org.objectweb.perseus.cache.replacement.lib.LRUReplacementManager;
import org.objectweb.perseus.cache.replacement.lib.MRUReplacementManager;
import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;

import junit.framework.TestCase;

/**
 * 
 *
 * @author chassase
 */
public class CacheHelper extends TestCase {

    static Logger logger = null;
	static LoggerFactory loggerFactory = null;

	static final int FIFO = 0;
	static final int LRU = 1;
	static final int MRU = 2;

	protected static final String LOGGER_NAME_PREFIX = "org.objectweb.perseus.cache";
	protected BasicCacheManager cm;

	

	public CacheHelper(String s) {
		super(s);
		if (logger == null) {
		    synchronized(CacheHelper.class) {
				if (logger == null) {
				    loggerFactory = Monolog.getMonologFactory();
				    logger = loggerFactory.getLogger(getLoggerName());
				}
		    }
		}
		cm = new BasicCacheManager();
	}

	protected String getLoggerName() {
	    return LOGGER_NAME_PREFIX + ".test";
	}
	
	protected void setUp(int type, int cacheSize, int autoCleanSize, int threshold) {
		setUp(type, cacheSize, autoCleanSize, threshold, new BasicCacheEntryFactory());
	}

	protected void setUp(int type, int cacheSize, int autoCleanSize, int threshold, CacheEntryFactory cef) {
		cm = new BasicCacheManager();
		cm.bindFc("logger", logger);
		cm.bindFc("monolog-factory", loggerFactory);
		cm.bindFc(BasicCacheManager.CACHE_ENTRY_FACTORY_BINDING, cef);
		AbstractReplacementManager rm;
		switch (type) {
		case FIFO:
			rm = new FIFOReplacementManager();
			break;
		case LRU:
			rm = new LRUReplacementManager();
			break;
		default:
			rm = new MRUReplacementManager();
			break;
		}
		cm.bindFc(BasicCacheManager.REPLACEMENT_MANAGER_BINDING, rm);
		cm.bindFc("logger", logger);
		cm.bindFc("monolog-factory", loggerFactory);
		rm.bindFc(AbstractReplacementManager.UNBIND_MANAGER_BINDING, cm);
		rm.bindFc("logger", logger);
		rm.bindFc("monolog-factory", loggerFactory);
		try {
			cm.setMaxObjects(cacheSize);
			cm.setAutoCleanSize("" + autoCleanSize);
			cm.setAutoCleanThreshold("" + threshold);
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

}
