/**
 * Copyright (C) 2001-2005 France Telecom R&D
 */
package org.objectweb.perseus.cache;

import org.objectweb.perseus.cache.api.CacheEntry;
import org.objectweb.perseus.cache.api.CacheEntryFactory;
import org.objectweb.perseus.cache.api.CacheException;
import org.objectweb.perseus.cache.api.CacheManager;
import org.objectweb.perseus.cache.api.FixableCacheEntry;
import org.objectweb.perseus.cache.lib.BasicReplaceableCacheEntry;
import org.objectweb.util.monolog.api.BasicLevel;


import junit.framework.Assert;


/**
 * 
 *
 * @author chassase
 */
public class TestCacheRobustness extends CacheHelper {

	public TestCacheRobustness(String s) {
		super(s);
	}

	protected String getLoggerName() {
	    return LOGGER_NAME_PREFIX + ".test.robustness";
	}
	public static class MyEntry extends BasicReplaceableCacheEntry {
	    public MyEntry(Object oid) {
	        super(oid);
	    }
	}
	
	public static class MyEntryFactory implements CacheEntryFactory {
        public FixableCacheEntry create(Object id, Object obj) {
           return (MyEntry) obj;
        }
	}
	

	public void _testBillon() {
		setUp(FIFO, 5000, 4800, 200);
		final int nbThread = 100;
		final CacheManager _cm = cm;
		final int nbActions = 10000; // must be greater than 100
		logger.log(BasicLevel.INFO, "Run concurrent Loading, " + nbThread + " threads");
		Thread[] ts = new Thread[nbThread];
		for (int i = 0; i < nbThread; i++) {
		    final int threadId = i;
			ts[i] = new Thread(
					new Runnable() {
						public void run() {
						    for(int j=0; j<nbActions; j++) {
							    Object o = new Integer(threadId * nbActions + j);
							    Assert.assertNull("Object " + o + " already in cache",_cm.lookup(o));
								try {
								    _cm.bind(o,o);
								} catch (CacheException e) {
									e.printStackTrace();
									fail(e.getMessage());
								}
								CacheEntry ce = _cm.lookup(o);
							    Assert.assertNotNull("Object " + o + " not in cache", ce);
							    Assert.assertEquals("Bad identifier", o, ce.getCeIdentifier());
							    Assert.assertEquals("Bad object", o, ce.getCeObject());
								if (j % (nbActions/100) == 0) {
								    logger.log(BasicLevel.DEBUG, "Thread " + threadId + " has done " + j + " actions.");
								}
						    }
						}
					}
			);
		}
		for (int i = 0; i < nbThread; i++) {
			ts[i].start();
		}
		try {
			for (int i = 0; i < nbThread; i++) {
				ts[i].join();
			}
		} catch (InterruptedException e) {
			fail(e.getMessage());
		}
	    
	}

	public void test100BillonObjectIsCE() {
		setUp(FIFO, 5000, 4800, 200, new MyEntryFactory());
		final int nbThread = 100;
		final CacheManager _cm = cm;
		final int nbActions = 1000000; // must be greater than 100
		logger.log(BasicLevel.INFO, "Run concurrent Loading, " + nbThread + " threads");
		Thread[] ts = new Thread[nbThread];
		for (int i = 0; i < nbThread; i++) {
		    final int threadId = i;
			ts[i] = new Thread(
					new Runnable() {
						public void run() {
						    for(int j=0; j<nbActions; j++) {
							    Object id = new Integer(threadId * nbActions + j);
						        MyEntry me = new MyEntry(id);
							    Assert.assertNull("Object " + id + " already in cache",_cm.lookup(id));
								try {
								    _cm.bind(id, me);
								} catch (CacheException e) {
									e.printStackTrace();
									fail(e.getMessage());
								}
								CacheEntry ce = _cm.lookup(id);
							    Assert.assertNotNull("Object " + id + " not in cache", ce);
							    Assert.assertEquals("Bad object", me, ce);
								if (j % (nbActions/100) == 0) {
								    logger.log(BasicLevel.WARN, "Thread " + threadId + " has done " + j + " actions.");
								}
						    }
						}
					}
			);
		}
		for (int i = 0; i < nbThread; i++) {
			ts[i].start();
		}
		try {
			for (int i = 0; i < nbThread; i++) {
				ts[i].join();
			}
		} catch (InterruptedException e) {
			fail(e.getMessage());
		}
	    
	}
}
